<?php

/**
 * @package crypto.lib
 * @author Valentin Schmidt
 * @version 1.1
 *
 * Simple string encryption/decryption functions that are compatible with
 * the Lingo function counterparts in script 'crypto.lib.ls'
 *
 * Crypto Xtra AES specs:
 * - AES-128 Encryption
 * - Mode: CBC
 * - IV prepended to encrypted data
 * - optional 16 byte header starts with 'AES', byte 5 contains pad_length
 *
 * Notice:
 * if you only want to encrypt strings, no binary data, i.e. no data that might contain chr(0),
 * you don't need a header with pad_length, you can simply right-trim the string returned by
 * decrypt_string() using: $decrypted = rtrim($decrypted, "\0");
 *
 */

define('AES_MAGIC',      'AES');
define('AES_HEADER_LEN', 12);
define('AES_BLOCK_LEN',  16);
define('AES_KEY_LEN',    16); // 16 for AES-128


/**
 * @param string $input 
 * @param string $key 16-32 bytes (alternatively use raw md5 checksum)
 * @param boolean $add_header Optional, default=false
 *
 * @return string Returns encrypted string (Base64 encoded)
 */
function encrypt_string($input, $key, $add_header=false){
  // $key = md5($key, true); // optional, to allow arbitrary key length
  
  srand((double) microtime() * 1000000); // for sake of MCRYPT_RAND
  $cp = mcrypt_module_open('rijndael-'.(AES_KEY_LEN*8), '', 'cbc', '');
  
  if (strtoupper(substr(PHP_OS, 0, 3)) === 'WIN')
    $iv = mcrypt_create_iv(mcrypt_enc_get_iv_size($cp), MCRYPT_RAND);
  else
    $iv = mcrypt_create_iv(mcrypt_enc_get_iv_size($cp), MCRYPT_DEV_RANDOM);
       
  // add header
  if ($add_header){    
    $pad_len = AES_BLOCK_LEN - (strlen($input)%AES_BLOCK_LEN);
    $encrypted = "AES".chr(AES_KEY_LEN)."\x00".chr($pad_len)."\x00\x00\x00\x00\x00\x00";
  }else{
    $encrypted = '';
  }

  mcrypt_generic_init($cp, $key, $iv);
  $encrypted .= $iv . mcrypt_generic($cp, $input);
  mcrypt_generic_deinit($cp);
  mcrypt_module_close($cp);
  
  return base64_encode($encrypted);
}

/**
 * @param string $input Encrypted string (Base64 encoded)
 * @param string $key 16-32 bytes (alternatively use raw md5 checksum)
 *
 * @return string Returns decrypted string
 */
function decrypt_string($input, $key){
  // $key = md5($key, true); // optional, to allow arbitrary key length
  
  $input = base64_decode($input);

  // parse header
  if (substr($input, 0, 3)==AES_MAGIC){
    $pad_len = ord($input{5});
    $data_start = AES_HEADER_LEN;
  }else{
    $pad_len = 0;
    $data_start = 0;
  }

  $cp = mcrypt_module_open('rijndael-'.(AES_KEY_LEN*8), '', 'cbc', '');

  $iv_size = mcrypt_enc_get_iv_size($cp);
  $iv = substr($input, $data_start, $iv_size);
  $input = substr($input, $data_start+$iv_size);

  mcrypt_generic_init($cp, $key, $iv);
  $decrypted = mdecrypt_generic($cp, $input);
  mcrypt_generic_deinit($cp);
  mcrypt_module_close($cp);

  if ($pad_len>0)
    $decrypted = substr($decrypted, 0, -$pad_len);

  return $decrypted;
}

?>